"""
Use case : Ishigami test function
=================================
"""
from __future__ import print_function
import openturns as ot
from typing import Any, List
import numpy as np
from dataclasses import dataclass
from dataclasses import field


@dataclass
class IshigamiModel():
    """Custom class for the Ishigami test function
    """
    dim: int = 3
    a: float = 7.0
    b: float = 0.1

    # X1
    X1: Any = ot.Uniform(-np.pi, np.pi)
    X1.setName("X1")

    # X2
    X2: Any = ot.Uniform(-np.pi, np.pi)
    X2.setName("X2")

    # X3
    X3: Any = ot.Uniform(-np.pi, np.pi)
    X3.setName("X1")

    distributionX = ot.ComposedDistribution([X1, X2, X3])
    distributionX.setDescription(['X1', 'X2', 'X3'])

    ishigami: Any = ot.SymbolicFunction(['X1', 'X2', 'X3', 'a', 'b'], ['sin(X1) + a * sin(X2)^2 + b * X3^4 * sin(X1)'])

    model: Any = ot.ParametricFunction(ishigami, [3, 4], [a, b])

    expectation: float = a / 2.0
    variance: float = 1.0/2 + a**2/8.0 + b*np.pi**4/5.0 + b**2*np.pi**8/18.0
    S1: float = (1.0/2.0 + b*np.pi**4/5.0+b**2*np.pi**8/50.0)/variance
    S2: float = (a**2/8.0)/variance
    S3: float = 0.0
    S12: float = 0.0
    S13: float = b**2*np.pi**8/2.0*(1.0/9.0-1.0/25.0)/variance
    S23: float = 0.0
    S123: float = 0.0
    ST1: float = S1 + S13
    ST2: float = S2
    ST3: float = S3 + S13



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

"""Deterministic meta-package."""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _base
else:
    import _base

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)



from openturns.common import PlatformInfo
def TESTPREAMBLE():
    """
    TESTPREAMBLE handles command-line arguments.

    Catch the --version argument on the command line,
    print a identification string and then exit softly.
    """
    import sys
    if '--version' in sys.argv:
        print(sys.argv[0] + ' ' + PlatformInfo.GetVersion())
        sys.exit(0)
    PlatformInfo.SetTwoDigitExponent()

class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _base.delete_SwigPyIterator

    def value(self):
        return _base.SwigPyIterator_value(self)

    def incr(self, n=1):
        return _base.SwigPyIterator_incr(self, n)

    def decr(self, n=1):
        return _base.SwigPyIterator_decr(self, n)

    def distance(self, x):
        return _base.SwigPyIterator_distance(self, x)

    def equal(self, x):
        return _base.SwigPyIterator_equal(self, x)

    def copy(self):
        return _base.SwigPyIterator_copy(self)

    def next(self):
        return _base.SwigPyIterator_next(self)

    def __next__(self):
        return _base.SwigPyIterator___next__(self)

    def previous(self):
        return _base.SwigPyIterator_previous(self)

    def advance(self, n):
        return _base.SwigPyIterator_advance(self, n)

    def __eq__(self, x):
        return _base.SwigPyIterator___eq__(self, x)

    def __ne__(self, x):
        return _base.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n):
        return _base.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n):
        return _base.SwigPyIterator___isub__(self, n)

    def __add__(self, n):
        return _base.SwigPyIterator___add__(self, n)

    def __sub__(self, *args):
        return _base.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _base:
_base.SwigPyIterator_swigregister(SwigPyIterator)


class TestFailed:
    """TestFailed is used to raise an uniform exception in tests."""

    __type = "TestFailed"

    def __init__(self, reason=""):
        self.reason = reason

    def type(self):
        return TestFailed.__type

    def what(self):
        return self.reason

    def __str__(self):
        return TestFailed.__type + ": " + self.reason

    def __lshift__(self, ch):
        self.reason += ch
        return self

import openturns.common
import openturns.typ
import openturns.statistics
import openturns.graph
import openturns.func
import openturns.geom
import openturns.diff
import openturns.optim
import openturns.experiment
import openturns.solver
import openturns.algo





"""
Use case : flood model
======================
"""
from __future__ import print_function
import openturns as ot
import openturns.viewer as viewer
from matplotlib import pylab as plt
from typing import Any, List
import numpy as np
from dataclasses import dataclass
from dataclasses import field


@dataclass
class FloodModel():
    """Custom class for the flood model.
    """
    dim: int = 4  # number of inputs
    # Q
    Q: Any = ot.TruncatedDistribution(ot.Gumbel(558., 1013.), 0, ot.TruncatedDistribution.LOWER)
    Q.setDescription("Q")
    Q.setName("Q")

    # Ks
    Ks: Any = ot.TruncatedDistribution(ot.Normal(30.0, 7.5), 0, ot.TruncatedDistribution.LOWER)
    Ks.setName("Ks")

    # Zv
    Zv: Any = ot.Uniform(49.0, 51.0)
    Zv.setName("Zv")

    # Zm
    Zm: Any = ot.Uniform(54.0, 56.0)
    #Zm.setDescription(["Zm (m)"])
    Zm.setName("Zm")

    model: Any = ot.SymbolicFunction(['Q', 'Ks', 'Zv', 'Zm'],
                                     ['(Q/(Ks*300.*sqrt((Zm-Zv)/5000)))^(3.0/5.0)+Zv-58.5'])

    distribution: Any = ot.ComposedDistribution([Q, Ks, Zv, Zm])
    distribution.setDescription(['Q', 'Ks', 'Zv', 'Zm'])

    X: Any = ot.RandomVector(distribution)
    X.setDescription(['Q', 'Ks', 'Zv', 'Zm'])

    Y: Any = ot.CompositeRandomVector(model, X)
    Y.setDescription('Y')
    event: Any = ot.ThresholdEvent(Y, ot.Greater(), 0.0)
    event.setName('overflow')



"""
Use case : logistic model
=========================
"""
from __future__ import print_function
import openturns as ot
from typing import Any, List
import numpy as np
from dataclasses import dataclass
from dataclasses import field


class Popu(ot.OpenTURNSPythonPointToFieldFunction):
    """
    """

    def __init__(self, t0=1790.0, tfinal=2000.0, nt=1000):
        grid = ot.RegularGrid(t0, (tfinal - t0) / (nt - 1), nt)
        super(Popu, self).__init__(3, grid, 1)
        self.setInputDescription(['y0', 'a', 'b'])
        self.setOutputDescription(['N'])
        self.ticks_ = [t[0] for t in grid.getVertices()]
        self.phi_ = ot.SymbolicFunction(['t', 'y', 'a', 'b'], ['a*y - b*y^2'])

    def _exec(self, X):
        y0 = X[0]
        a = X[1]
        b = X[2]
        phi_ab = ot.ParametricFunction(self.phi_, [2, 3], [a, b])
        phi_t = ot.ParametricFunction(phi_ab, [0], [0.0])
        solver = ot.RungeKutta(phi_t)
        initialState = [y0]
        values = solver.solve(initialState, self.ticks_)
        return values * [1.0e-6]


@dataclass
class LogisticModel():
    """Custom class for the logistic model
    """

    y0: float = 3.9e6
    a: float = 0.03134
    b: float = 1.5887e-10
    distY0: Any = ot.Normal(y0, 0.1 * y0)
    distA: Any = ot.Normal(a, 0.3 * a)
    distB: Any = ot.Normal(b, 0.3 * b)
    distX: Any = ot.ComposedDistribution([distY0, distA, distB])

    F: Any = Popu(1790.0, 2000.0, 1000)
    popu: Any = ot.PointToFieldFunction(F)



"""
Use case : Ackley test function
===============================
"""
from __future__ import print_function
import openturns as ot
from typing import Any, List
import numpy as np
from dataclasses import dataclass
from dataclasses import field

# model


def g(X):
    a = 20.0
    b = 0.2
    c = 2.0 * np.pi
    d = len(X)
    sumOfSquared = sum(x**2 for x in X) / d
    sumOfCos = sum(np.cos(c * x) for x in X) / d
    f = - a * np.exp(- b * np.sqrt(sumOfSquared)) \
        - np.exp(sumOfCos) + a + np.exp(1.0)
    return [f]


@dataclass
class AckleyModel():
    """Custom class for the Ackley test function.
    """
    dim: int = 2
    a = 20.0
    b = 0.2
    c = 2.0 * np.pi

    model: Any = ot.PythonFunction(dim, 1, g)

    # Bounds
    lowerbound: Any = ot.Point([-15.0] * dim)
    upperbound: Any = ot.Point([15.0] * dim)

    # The Ackley function has many local minimas
    # The global minimum is unique and located at the center of the domain
    x0: List[float] = field(default_factory=list)

    def __post_init__(self):
        self.x0 = [0.0] * self.dim



"""
Use case : vertical deflection of a tube
========================================
"""
from __future__ import print_function
import openturns as ot
from typing import Any, List
import numpy as np
from dataclasses import dataclass
from dataclasses import field


@dataclass
class DeflectionTube():
    """Custom class for the deflection of a tube
    """
    dim: int = 6

    model: Any = ot.SymbolicFunction(["F", "L", "a", "De", "di", "E"], ["g1", "g2", "g3"], "var I:=pi_*(De^4-di^4)/32; var b:=L-a; g1:=-F*a^2*(L-a)^2/(3*E*L*I); g2:=-F*b*(L^2-b^2)/(6*E*L*I); g3:=F*a*(L^2-a^2)/(6*E*L*I)")
    model.setOutputDescription(["Deflection", "Left angle", "Right angle"])

    XF: Any = ot.Normal(1, 0.1)
    XF.setDescription(["Force"])

    XE: Any = ot.Normal(200000, 2000)
    XE.setDescription(["Young Modulus"])

    XL: Any = ot.Dirac(1.5)
    XL.setDescription(["Length"])

    Xa: Any = ot.Dirac(1.0)
    Xa.setDescription(["Location"])

    XD: Any = ot.Dirac(0.8)
    XD.setDescription(["External diameter"])

    Xd: Any = ot.Dirac(0.1)
    Xd.setDescription(["Internal diameter"])

    inputDistribution: Any = ot.ComposedDistribution([XF, XL, Xa, XD, Xd, XE])
